#include "CarnahanStarling.h"


CarnahanStarling::CarnahanStarling(double phi, double beta)
{
  this->phi = phi;
  this->rho = (this->phi * (6.0 /  M_PI));
  this->beta = beta;
}

/************************************************************************
*                                                                       *
*   Compressibility of fluid                                            *
*	  drho/dp = beta / (Z + rho * dZ/drho)                              *
*                                                                       *
************************************************************************/

double CarnahanStarling::getCompressibility(void) 
{
	double comp = this->beta / (this->getZ() + this->phi * this->getdZdphi());
//	cout << comp << "\t";
	return(comp);
}

/************************************************************************
*                                                                       *
*	Free Energy of Hard Sphere Fluid                                    *
*	  A = A_ideal + A_excess                                            *
*	      A_ideal = log(rho)-1                                          *
*                                                                       *
************************************************************************/

double CarnahanStarling::getAHS(void) 
{
	double AHS = (this->getAexHS() + log(this->rho) - 1);
//	cout << AHS << "\t";
	return(AHS);
}

/************************************************************************
*                                                                       *
*   simple liquids p. 96 from carnahan starling (Eqn 4.6.15)            *
*                                                                       *
*   Beta *  Aex / N    ==    phi * (4 - 3*phi) / ((1-phi) ^ 2)          *
*                                                                       *
************************************************************************/

double CarnahanStarling::getAexHS(void) 
{
	double AexHS = (4.0 - (3.0 * this->phi));
	AexHS /= pow((1.0 - this->phi), 2.0);
	AexHS *= this->phi;
//	cout << AexHS << "\t";
	return(AexHS);
}

/************************************************************************
*                                                                       *
*   Carnahan Starling equation of state                                 *
*      simple liquids p. 95 (Eqn 4.6.14)                                *
*                                                                       *
*   Z = (beta * P / phi) = (1 + phi + phi^2 - phi^3) / (1-phi)^3        *
*                                                                       *
************************************************************************/

double CarnahanStarling::getZ(void) 
{
	return ( (1.0 + this->phi + pow(this->phi,2.0) - pow(this->phi,3.0))
			/ pow( (1.0 - this->phi), 3.0)
		);
}

/************************************************************************
*                                                                       *
*   Analytical Derivative of Carnahan Starling equation of state        *
*                                                                       *
*   Z =  (1.0 + phi + phi^2 - phi^3) / (3 * (1-phi)^4)                  *
*        + (1.0 + 2phi - 3phi^2) / (1-phi)^3                            *
*                                                                       *
************************************************************************/

double CarnahanStarling::getdZdphi(void) 
{
	double derivative_term1 = ( 1.0 + this->phi + pow(this->phi,2.0) - pow(this->phi,3.0) );
	derivative_term1 *= ( 3.0 * pow((1.0 - this->phi),-4.0) );

	double derivative_term2 = (1.0 + (2.0 * this->phi) - (3.0 * pow(this->phi,2.0)) );
	derivative_term2 *= pow((1.0 - this->phi), -3.0);

	return(derivative_term1 + derivative_term2);
}

